`timescale 1ns / 1ns

/** 
 * This module implements a very simple first-order low-pass filter
 * with coefficients = 0.5, 0.25, and 0.25
 * 
 * Signals:
 * 
 * CLK       master clock
 * 
 * RST       active high reset
 * 
 * ENABLE    signal to calculate the next sample
 * 
 * DATA_IN   incoming sample.  Must be valid at assertion of ENABLE
 * 
 * DATA_OUT  outgoing sample from previous assertion of ENABLE
 */

module bridge_filter (CLK, RST, ENABLE, DATA_IN, DATA_OUT);

   input CLK;
   input RST;
   input ENABLE;
   
   wire  CLK;
   wire  RST;
   wire  ENABLE;

   input [0:15] DATA_IN;
   output [0:15] DATA_OUT;

   wire [0:15] 	 DATA_IN;
   reg [0:15] 	 DATA_OUT;
   
   reg [0:15] 	 curr_input;
   reg [0:15] 	 prev_input;
   reg [0:15] 	 prev_prev_input;

   
   // there're an extra 2 bits here so that I can do the divide by 4 afterwards
   wire [0:17] 	 raw_data_out;
   wire [0:15] 	 pull_towards_zero;
   
   // this funny notation is to cause it to sign extend the
   // values
   assign 		 raw_data_out = 
				 {curr_input[0], curr_input[0:15], 1'b0} +
				 {{2{prev_prev_input[0]}}, prev_prev_input[0:15]}  +
				 {{2{prev_input[0]}}, prev_input[0:15]}  ;

   // 1/128 of the value so that it gets pulled gently back to 0 ( or to within 127 of 0)
   assign 		 pull_towards_zero = {{7{raw_data_out[0]}}, raw_data_out[0:8]};
		 

   always @(posedge CLK) 
	 begin
		if (RST) 
		  begin
			 prev_prev_input <= 0;
			 prev_input <= 0;
			 curr_input <= 0;
			 DATA_OUT <= 0;
		  end
		else 
		  begin
			 if (ENABLE) 
			   begin
				  prev_prev_input <= prev_input;
				  prev_input <= curr_input;
				  curr_input <= DATA_IN;
				  DATA_OUT <= pull_towards_zero - raw_data_out[0:15];
			   end
		  end // else: !if(RST)
	 end
   
endmodule
