`timescale 1ns / 1ns

/** codec_transmitter
 * transmits a left and right channel to the sound codec.  Even though
 * the codec is technically 24-bit, this is only 16 bit and it pads the
 * least significant 8 bits with 0s
 * 
 * Signals
 * 
 * RST   active high reset
 * 
 * CLK   master clock
 * 
 * BICK  bit clock for codec
 * 
 * LRCK  sample clock for codec
 * 
 * SDOUT serial line connected to codec
 * 
 * DATA  data to send to the codec.  it is grabbed at the rising edge of 
 *       LRCK
 */
module codec_transmitter(RST, CLK,
                         BICK, LRCK,
                         SDOUT, DATA);

   parameter WAIT_FOR_RISING_LRCK   = 0,
             WRITING_LEFT           = 1,
             WAIT_FOR_FALLING_LRCK  = 2,
             WRITING_RIGHT          = 3;
   
             
   input RST;
   input CLK;
   input BICK;
   input LRCK;
   input [0:31] DATA;
   
   wire  RST;
   wire  CLK;
   wire  BICK;
   wire  LRCK;
   wire [0:31] DATA;


   
   output      SDOUT;
   wire        SDOUT;
   
   
   
   // internal state
   reg 		   bick_buf, prev_bick, prev_lrck;
   wire 	   falling_bick, falling_lrck, rising_lrck;
   reg [0:3]   counter, next_counter;
   reg [0:1]   state, next_state;
   reg [0:31]  shift_reg, next_shift_reg;
   
   
   assign 	   falling_bick = ~bick_buf && prev_bick;
   assign 	   falling_lrck = ~LRCK && prev_lrck;
   assign 	   rising_lrck = LRCK && ~prev_lrck;
   
   
   assign 	   SDOUT = (state == WRITING_LEFT || state == WRITING_RIGHT) ? shift_reg[0] : 0;
   
   
   always @(posedge CLK)
     begin
		bick_buf <= BICK;
		prev_bick <= bick_buf;
        prev_lrck <= LRCK;
        
        if(RST)
          begin
             counter <= 0;
             state <= 0;
             shift_reg <= 0;
          end
        else
          begin
             counter <= next_counter;
             state <= next_state;
             shift_reg <= next_shift_reg;
          end // else: !if(RST)
     end // always @ (posedge CLK)

   always @(*)
     begin
        next_counter = counter;
        next_state = state;
        next_shift_reg = shift_reg;
        case(state)
          WAIT_FOR_RISING_LRCK:
            begin
               if(rising_lrck)
                 begin
                    next_state = WRITING_LEFT;
                    next_counter = 0; // this should coincide with the falling bick
                    next_shift_reg = DATA;
                 end
            end
          WRITING_LEFT:
            begin
               if(falling_bick)
                 begin
                    next_counter = counter + 1;
                    next_shift_reg = {shift_reg[1:31], 1'b0};
                    if(counter == 15)
                      next_state = WAIT_FOR_FALLING_LRCK;
                 end
            end
          WAIT_FOR_FALLING_LRCK:
            begin
               if(falling_lrck)
                 begin
                    next_counter = 0;
                    next_state = WRITING_RIGHT;
                 end
            end
          WRITING_RIGHT:
            begin
               if(falling_bick)
                 begin
                    next_counter = counter + 1;
                    next_shift_reg = {shift_reg[1:31], 1'b0};
                    if(counter == 15)
                      next_state = WAIT_FOR_RISING_LRCK;
                 end
            end
        endcase // case(state)
     end // always @ (*)
endmodule // codec_receiver

     