`timescale 1ns / 1ns

/** delay_line
 * A simple delay line with only a single input and a single output.  The
 * length is variable, and it will output garbage until it receives
 * SIZE samples.
 * 
 * Signals
 * CLK          master clock
 * 
 * RST          active high reset
 * 
 * SIZE         number of samples held by this delay line
 * 
 * ENABLE       signal that it should move on to the next sample
 * 
 * SAMPLE_IN    value to insert into the delay line
 * 
 * SAMPLE_OUT   value coming out the other end of the delay line
 */

module delay_line(CLK, RST, SIZE, ENABLE,
				  SAMPLE_IN, SAMPLE_OUT);
   
   input 	   CLK;
   input 	   RST;
   input [0:7] SIZE;
   input 	   ENABLE;

   wire 	   CLK;
   wire 	   RST;
   wire [0:7]  SIZE;
   wire 	   ENABLE;

   
   input [0:15]  SAMPLE_IN;
   output [0:15] SAMPLE_OUT;

   wire [0:15]   SAMPLE_IN;
   wire [0:15]   SAMPLE_OUT;
   
   reg [0:7]     curr_end_pos;

   
   function [0:7] next_end_pos;
      input [0:7] curr_end_pos;
      input [0:7] size;
      reg [0:7] potential_end_pos;
      begin
         potential_end_pos = curr_end_pos + 1;
         next_end_pos = (potential_end_pos >= size) ? 0 : potential_end_pos;
      end
   endfunction // next_end_pos

      
   always @(posedge CLK) 
	 begin
		if (RST) 
		  begin
			 curr_end_pos <= 0;
          end 
		else
		  begin
			 if (ENABLE) 
			   begin
				  curr_end_pos <= next_end_pos(curr_end_pos, SIZE);
			   end
		  end // else: !if(RST)
	 end // always @ (posedge CLK)


   ram_block rb
     (
      .CLK(CLK),
      .WE(ENABLE),
      .A(curr_end_pos),
      .DI(SAMPLE_IN),
      .DO(SAMPLE_OUT)
      );
   
endmodule // delay_line
