`timescale 1ns / 1ns

/** delay_line_with_pickup
 * Implements a delay line with a variable pickup location
 * 
 * Signals
 * CLK           master clock
 * 
 * RST           active high reset
 * 
 * SIZE          the total length of the delay line
 * 
 * ENABLE        signal that the delay line should move ahead one sample
 * 
 * PICKUP_LOC    location of the pickup.  Must be under 255 and 
 *               SIZE - PICKUP_LOC must be under 255
 * 
 * PICKUP_VALUE  value of the delay line at the pickup location
 * 
 * SAMPLE_IN     sample to add to the end of the delay line
 * 
 * SAMPLE_OUT    sample coming off the end of the delay line
 */

module delay_line_with_pickup(CLK, RST, SIZE, ENABLE,
							  PICKUP_LOC, PICKUP_VALUE,
							  SAMPLE_IN, SAMPLE_OUT);

   input       CLK;
   input       RST;
   input [0:8] SIZE;
   input 	   ENABLE;

   wire 	   CLK;
   wire 	   RST;
   wire [0:8]  SIZE;
   wire 	   ENABLE;


   
   input [0:7] PICKUP_LOC;
   output [0:15] PICKUP_VALUE;

   wire [0:7] 	 PICKUP_LOC;
   wire [0:15] 	 PICKUP_VALUE;


   
   input [0:15]  SAMPLE_IN;
   output [0:15] SAMPLE_OUT;

   wire [0:15] 	 SAMPLE_IN;
   wire [0:15] 	 SAMPLE_OUT;


   
   // signals for before the pickup
   wire 		 pre_pickup_clk;
   wire 		 pre_pickup_rst;
   wire [0:7] 	 pre_pickup_size;
   wire 		 pre_pickup_enable;
   wire [0:15] 	 pre_pickup_sample_in;
   wire [0:15] 	 pre_pickup_sample_out;

   // signals for after the pickup
   wire 		 post_pickup_clk;
   wire 		 post_pickup_rst;
   wire [0:7] 	 post_pickup_size;
   wire 		 post_pickup_enable;
   wire [0:15] 	 post_pickup_sample_in;
   wire [0:15] 	 post_pickup_sample_out;


   assign 		 pre_pickup_clk = CLK;
   assign 		 pre_pickup_rst = RST;
   assign 		 pre_pickup_size = PICKUP_LOC;
   assign 		 pre_pickup_enable = ENABLE;
   assign 		 pre_pickup_sample_in = SAMPLE_IN;

   assign 		 post_pickup_clk = CLK;
   assign 		 post_pickup_rst = RST;
   assign 		 post_pickup_size = SIZE - PICKUP_LOC;
   assign 		 post_pickup_enable = ENABLE;
   assign 		 post_pickup_sample_in = pre_pickup_sample_out;

   // io assigns
   assign 		 PICKUP_VALUE = pre_pickup_sample_out;
   assign 		 SAMPLE_OUT = post_pickup_sample_out;


   delay_line pre_pickup
	 (
	  .CLK(pre_pickup_clk),
	  .RST(pre_pickup_rst),
	  .SIZE(pre_pickup_size),
	  .ENABLE(pre_pickup_enable),
	  .SAMPLE_IN(pre_pickup_sample_in),
	  .SAMPLE_OUT(pre_pickup_sample_out)
	  );

   delay_line post_pickup
	 (
	  .CLK(post_pickup_clk),
	  .RST(post_pickup_rst),
	  .SIZE(post_pickup_size),
	  .ENABLE(post_pickup_enable),
	  .SAMPLE_IN(post_pickup_sample_in),
	  .SAMPLE_OUT(post_pickup_sample_out)
	  );

endmodule   