`timescale 1ns / 1ns

/** impulse_generator
 * generates SIZE samples with a spike at PEAK_POS.  You might want to 
 * change this to make a different shape of wave.
 * 
 * Signals:
 * CLK             master clock
 * 
 * RST             active high reset
 * 
 * START_STROBE    strobe to signal the component to start generating
 *                 the signals with a spike at PEAK_POS
 * 
 * BUSY            signal that the component is currently busy.  It will
 *                 go low when it has output SIZE samples with the desired
 *                 spike
 * 
 * SIZE            the number of samples to output
 * 
 * PEAK_POS        the position of the spike in the set of samples output
 * 
 * PEAK_VAL        the value that the spike should be
 * 
 * OUTPUT_ENABLE   signal that it is outputting a value during this clock
 *                 cycle
 * 
 * OUTPUT          the value that is being output
 */

module impulse_generator(CLK, RST,
						 START_STROBE, BUSY,
						 SIZE, PEAK_POS, PEAK_VAL,
						 OUTPUT_ENABLE, OUTPUT);

   parameter STATE_READY           = 0;
   parameter STATE_WRITING         = 1;
   
   input CLK;
   input RST;

   wire  CLK;
   wire  RST;


   input START_STROBE;
   output BUSY;

   wire   START_STROBE;
   wire   BUSY;

   

   input [0:8] SIZE;
   input [0:8] PEAK_POS;
   input [0:15] PEAK_VAL;
   
   wire [0:8] 	SIZE;
   wire [0:8] 	PEAK_POS;
   wire [0:15] 	PEAK_VAL;

   // buffers just in case the values change while I'm working
   reg [0:8] 	size_buf, next_size_buf;
   reg [0:8] 	peak_pos_buf, next_peak_pos_buf;
   reg [0:15] 	peak_val_buf, next_peak_val_buf;


   
   output 		OUTPUT_ENABLE;
   output [0:15] OUTPUT;

   wire 		 OUTPUT_ENABLE;
   reg [0:15] 	 OUTPUT, next_OUTPUT;


   // internal signals
   reg 			 state, next_state;
   reg [0:8] 	 curr_pos, next_pos;
   

   
   assign 		 OUTPUT_ENABLE = (state == STATE_WRITING);
   assign 		 BUSY = (state != STATE_READY);
   		  

   always @(posedge CLK)
	 begin
		if(RST)
		  begin
			 size_buf <= 0;
			 peak_pos_buf <= 0;
			 peak_val_buf <= 0;
			 OUTPUT <= 0;
			 state <= STATE_READY;
			 curr_pos <= 0;
		  end // if (RST)
		else
		  begin
			 size_buf <= next_size_buf;
			 peak_pos_buf <= next_peak_pos_buf;
			 peak_val_buf <= next_peak_val_buf;
			 OUTPUT <= next_OUTPUT;
			 state <= next_state;
			 curr_pos <= next_pos;
		  end // else: !if(RST)
	 end // always @ (posedge CLK)


   always @(*)
	 begin
		next_state = state;
		case(state)
		  STATE_READY: 
			if(START_STROBE) next_state = STATE_WRITING;
		  STATE_WRITING:
			if(curr_pos >= size_buf)
			  next_state = STATE_READY;
		endcase // case(state)
	 end // always @ (*)



   always @(*)
	 begin
		next_size_buf = size_buf;
		next_peak_pos_buf = peak_pos_buf;
		next_peak_val_buf = peak_val_buf;
		if(state == STATE_READY)
		  begin
			 if(START_STROBE)
			   begin
				  next_size_buf = SIZE;
				  next_peak_pos_buf = PEAK_POS;
				  next_peak_val_buf = PEAK_VAL;
			   end
		  end
	 end // always @ (*)


   always @(*)
	 begin
		next_pos = curr_pos;
		next_OUTPUT = OUTPUT;
		case(state)
		  STATE_READY: next_pos = 0;
		  STATE_WRITING:
			begin
			   next_pos = curr_pos + 1;
			   // the output is 0 except for when we hit the
			   // peak position.
			   next_OUTPUT = (curr_pos == peak_pos_buf) ? peak_val_buf : 0;
			end
		endcase
	 end // always @ (*)

endmodule // impulse_generator

			  
		