`timescale 1ns / 1ns

/**
 * Top level for the opb_string_simulation core for use with the 
 * Xilinx EDK.  This module implements the functionality of 
 * 4 independent string simulations that can be controlled by the
 * Microblaze through memory mapped I/O.  The address offsets to the
 * individual registers can be found in the simulation_control.v 
 * file.  This module also includes all the hardware needed for 
 * driving the AK audio codec chip.
 * 
 * Signals:
 * 
 * The OPB and Sln signals are as described in the OPB spec.
 * 
 * The remaining signals are as described by the AK documentation.
 * They should be connected to the appropriate io pins on the 
 * AFX board.
 */
module opb_string_simulation(OPB_ABus,
                             OPB_BE,
                             OPB_Clk,
                             OPB_DBus,
                             OPB_RNW,
                             OPB_Rst,
                             OPB_select,
                             OPB_seqAddr,
                             
                             Sln_DBus,
                             Sln_errAck,
                             Sln_retry,
                             Sln_toutSup,
                             Sln_xferAck,
                             
                             MCLK,
                             BICK,
                             LRCK,
                             SDTI1,
                             SDTI2,
                             SDTI3,
                             SDTI4,
                             SDTO,
                             PDN);
   
   // begin edk parameters
   parameter C_BASEADDR      = 32'hFFFFFF00;
   parameter C_HIGHADDR      = 32'hFFFFFFFF;
   parameter C_USER_ID_CODE  = 3;
   parameter C_OPB_AWIDTH    = 32;
   parameter C_OPB_DWIDTH    = 32;
   // end edk parameters

   // begin opb signals
   input [0:31] OPB_ABus;
   input [0:3]  OPB_BE;
   input        OPB_Clk;
   input [0:31] OPB_DBus;
   input        OPB_RNW;
   input        OPB_Rst;
   input        OPB_select;
   input        OPB_seqAddr;
   
   wire [0:31]  OPB_ABus;
   wire [0:3]   OPB_BE;
   wire         OPB_Clk;
   wire [0:31]  OPB_DBus;
   wire         OPB_RNW;
   wire         OPB_Rst;
   wire         OPB_select;
   wire         OPB_seqAddr;
   
   reg [0:31]   OPB_ABus_Reg;
   reg [0:3]    OPB_BE_Reg;
   reg [0:31]   OPB_DBus_Reg;
   reg          OPB_RNW_Reg;
   reg          OPB_select_Reg;
   reg          OPB_seqAddr_Reg;
   
   output [0:31] Sln_DBus;
   output        Sln_errAck;
   output        Sln_retry;
   output        Sln_toutSup;
   output        Sln_xferAck;
   
   wire [0:31]   Sln_DBus;
   wire          Sln_errAck;
   wire          Sln_retry;
   wire          Sln_toutSup;
   wire          Sln_xferAck;
   // end opb signals

   
   // begin codec interface signals
   output        MCLK;
   output        BICK;
   output        LRCK;
   output        SDTI1;
   output        SDTI2;
   output        SDTI3;
   output        SDTI4;
   input         SDTO;
   output        PDN;


   wire          MCLK;
   wire          BICK;
   wire          LRCK;
   wire          SDTI1;
   wire          SDTI2;
   wire          SDTI3;
   wire          SDTI4;
   wire          SDTO;
   wire          PDN;
   // end codec interface signals

   
   wire          its_for_me;
   wire [0:31]   data_bus;
   wire          ack;
   
   
   
   
   assign        Sln_DBus = ((its_for_me && OPB_RNW_Reg) ? 
                             data_bus : 
                             32'h00000000);   
   
   assign        Sln_errAck = (its_for_me         && 
                               (OPB_BE_Reg != 4'hf ||
                                OPB_ABus_Reg[30:31] != 2'b00));
   
   assign        Sln_retry = 0;
   assign        Sln_toutSup = 0;
   assign        Sln_xferAck = its_for_me && ack;
   
   
   assign        its_for_me = (!OPB_Rst                              &&
                               OPB_select_Reg                        &&	
   							   OPB_select                            &&
                               C_BASEADDR[31:8] == OPB_ABus_Reg[0:23]);

   
   
   always @(posedge OPB_Clk)
     begin
        if(OPB_Rst)
          begin
             OPB_ABus_Reg <= 0;
             OPB_BE_Reg <= 0;
             OPB_DBus_Reg <= 0;
             OPB_RNW_Reg <= 0;
             OPB_select_Reg <= 0;
             OPB_seqAddr_Reg <= 0;
          end // if (OPB_Rst)
        else
          begin
             OPB_ABus_Reg <= OPB_ABus;
             OPB_BE_Reg <= OPB_BE;
             OPB_DBus_Reg <= OPB_DBus;
             OPB_RNW_Reg <= OPB_RNW;
             OPB_select_Reg <= OPB_select;
             OPB_seqAddr_Reg <= OPB_seqAddr;
          end // else: !if(OPB_Rst)
     end // always @ (posedge OPB_Clk)
   

   

   string_simulation ss
     (
      .CLK(OPB_Clk),
      .RST(OPB_Rst),

      .CTRL_EN(its_for_me),
      .CTRL_ADDR(OPB_ABus_Reg[24:31]),
      .CTRL_DATA_IN(OPB_DBus_Reg),
      .CTRL_DATA_OUT(data_bus),
      .CTRL_RNW(OPB_RNW_Reg),
      .CTRL_ACK(ack),
      
      .MCLK(MCLK),
      .BICK(BICK),
      .LRCK(LRCK),
      .SDTI1(SDTI1),
      .SDTI2(SDTI2),
      .SDTI3(SDTI3),
      .SDTI4(SDTI4),
      .SDTO(SDTO),
      .PDN(PDN)
      );

      
   
endmodule // opb_audio_controller

      