`timescale 1ns / 1ns

/*
 * simulation_control.v
 * 
 * This is the main controller for the opb_string_simulation.  It is the 
 * module that decodes instructions from the Microblaze and then sets the 
 * appropriate control lines in order to run the string simulation.  It also
 * can report back to the Microblaze on the current state of the various
 * values such as string length, pickup location, etc.  This interaction is 
 * handled through a memory mapped i/o.  
 * 
 * The first stage for decoding the transaction is taken care of by the
 * opb_string_simulation module.  There, it figures out if the transaction
 * was destined for this component, and then translates the address from the
 * 32-bit global address to an 8-bit internal address.  The internal address
 * is then used by this module to address the apropriate control and either 
 * update that control's value, or to report back on the value of the given
 * control. The internal addresses (in hex) are as follows: (note that X means
 * that the bits are ignored when decoding the address, so for example the
 * address 00 maps to the same control as the address 04)
 * 
 * 0X: control register.    The least significant bit is the master enable
 *                          the remaining bits are ignored during writes, but 
 *                          always read as 0's
 * 
 * 1X: pluck register.      Writing ones to this register causes the strings to 
 *                          pluck as follows:
 *                           bit 28: pluck string0
 *                           bit 29: pluck string1
 *                           bit 30: pluck string2
 *                           bit 31: pluck string3
 *                          The other bits in the word are ignored.  Writing a 0 
 *                          to any of the aforementioned bits has no effect.
 *                          reading this location always yields 0.
 *
 * 3X: dummy register       A location that acts just like ram.  You can write 
 *                          to this location and subsequent reads should return
 *                          the last written value.
 * 
 * 40: string0 length       The lower order 9 bits are the lengths of the 
 * 44: string1 length       given strings.  The other bits are ignored on 
 * 48: string2 length       writes and are read as zeroes
 * 4C: string3 length
 * 
 * 50: string0 pickup loc   The lower order 8 bits are the pickup locs of the 
 * 54: string1 pickup loc   given strings.  The other bits are ignored on 
 * 58: string2 pickup loc   writes and are read as zeroes
 * 5C: string3 pickup loc
 * 
 * 60: string0 pluck val    The lower order 16 bits are the pluck vals of the 
 * 64: string1 pluck val    given strings.  The other bits are ignored on 
 * 68: string2 pluck val    writes and are read as zeroes
 * 6C: string3 pluck val
 * 
 * 70: string0 pluck loc    The lower order 9 bits are the pluck locs of the 
 * 74: string1 pluck loc    given strings.  The other bits are ignored on 
 * 78: string2 pluck loc    writes and are read as zeroes
 * 7C: string3 pluck loc
 * 
 * Signals:
 * 
 * CLK               The master clock, probably running at 50MHz.  All external
 *                   events happen on the rising edge of this clock.
 * 
 * RST               Active high reset.
 * 
 * CTRL_EN           Active high signal that there is a command to either read
 *                   or write a value.
 * 
 * CTRL_ADDR         Address associated with the command.  This value must be
 *                   a multiple of 4 (last two bits are 0).  The bits are 
 *                   divided as follows:
 *                     [0:3] select bits:  Tell you what you're reading
 *                                         or writing.
 *                     [4:5] string bits:  Tell you which string it refers to
 *                     [6:7] zeroes:       must always be 0.
 * 
 * CTRL_DATA_IN      If it's a write operation, this bus carrys the data to be
 *                   written.  It should be ignored otherwise.
 * 
 * CTRL_DATA_OUT     If it's a read operation, these bits should be driven to 
 *                   the value that is being read.  These bits must be 0 at
 *                   all other times.
 * 
 * CTRL_RNW          read_not_write.  Tells if the operation is a read or write
 *                   operation.  If it is high, this is a read, otherwise it is
 *                   a write.
 * 
 * CTRL_ACK          acknowledge of the command.  This signal acknowledges the
 *                   command and states that if it was a read command, the 
 *                   value is now on the CTRL_DATA_OUT line.  If it was a write
 *                   command, the value that was on CTRL_DATA_IN has been 
 *                   successfully latched.
 * 
 * 
 *    NOTE: The signals below here are the control signals and they are polled
 *          repeatedly by the string simulation components
 * 
 * MASTER_ENABLE     the signal telling the string simulation whether it should
 *                   be running or not.  
 * 
 * STRING_LENGTHS    the lengths of the strings.  Note that it is formatted as 
 *                   follows:
 *                    [0:8]   string0
 *                    [9:17]  string1
 *                    [18:26] string2
 *                    [27:35] string3
 * 
 * PICKUP_LOCATIONS  the pickup locations of each string.  Note that it is
 *                   formatted as follows:
 *                    [0:7]   string0
 *                    [8:15]  string1
 *                    [16:23] string2
 *                    [24:31] string3
 * 
 * PLUCK_VALUES      the values of the plucks on each string.  Formatted as 
 *                   follows:
 *                    [0:15]  string0
 *                    [16:31] string1
 *                    [32:47] string2
 *                    [48:63] string3
 * 
 * PLUCK_LOCATIONS   where to pluck each string.  Formatted as follows:
 *                    [0:8]   string0
 *                    [9:17]  string1
 *                    [18:26] string2
 *                    [27:35] string3
 * 
 * PLUCK_STROBES     the strobe signals telling the strings to pluck.  These
 *                   values must be held high for 1 to 2 cycles when the write
 *                   address is written, but then they must return to 0.  The 
 *                   bus is formatted as follows
 *                    [0]     string0
 *                    [1]     string1
 *                    [2]     string2
 *                    [3]     string3
 * 
 */
module simulation_control(CLK,
                          RST,

                          CTRL_EN,
                          CTRL_ADDR,
                          CTRL_DATA_IN,
                          CTRL_DATA_OUT,
                          CTRL_RNW,
                          CTRL_ACK,

                          MASTER_ENABLE,
                          STRING_LENGTHS,
                          PICKUP_LOCATIONS,
                          PLUCK_VALUES,
                          PLUCK_LOCATIONS,
                          PLUCK_STROBES);

   input          CLK;
   input          RST;
   

   input          CTRL_EN;
   input [0:7]    CTRL_ADDR;
   input [0:31]   CTRL_DATA_IN;
   output [0:31]  CTRL_DATA_OUT;
   input          CTRL_RNW;
   output         CTRL_ACK;

   output         MASTER_ENABLE;
   output [0:35]  STRING_LENGTHS;
   output [0:31]  PICKUP_LOCATIONS;
   output [0:63]  PLUCK_VALUES;
   output [0:35]  PLUCK_LOCATIONS;
   output [0:3]   PLUCK_STROBES;


   wire           CTRL_EN;
   wire [0:7]     CTRL_ADDR;
   wire [0:31]    CTRL_DATA_IN;
   reg [0:31]     CTRL_DATA_OUT;
   wire           CTRL_RNW;
   reg            CTRL_ACK;
   
   reg            MASTER_ENABLE;
   reg [0:35]     STRING_LENGTHS;
   reg [0:31]     PICKUP_LOCATIONS;
   reg [0:63]     PLUCK_VALUES;
   reg [0:35]     PLUCK_LOCATIONS;
   reg [0:3]      PLUCK_STROBES;

   
   // this register isn't really part of the simulation, but it
   // is probably a good first step to make sure you understand
   // how to read and write based on the addresses.
   reg [0:31]     dummy_register;
   

   /*
    * here are a bunch of utility functions for extracting and
    * setting the individual strings' associated values.  These are
    * provided because all of the strings' values are embedded inside the
    * a single register.  It's just that the 0'th string is the first
    * set of bits, the 1'th string is the next set and so forth.  These
    * functions abstract this out a little bit.
    * 
    * You call functions in verilog just like you do in C.
    * 
    * An example usage:
    * 
    *   reg [0:1] the_string_index;
    *   reg [0:8] the_length;
    * 
    *   ... 
    * 
    *   the_length <= get_string_length(the_string_index);
    * 
    */
   
   function [0:8] get_string_length;
      input [0:1] which_string;
      begin
         case(which_string)
           2'b00: get_string_length = STRING_LENGTHS[0:8];
           2'b01: get_string_length = STRING_LENGTHS[9:17];
           2'b10: get_string_length = STRING_LENGTHS[18:26];
           2'b11: get_string_length = STRING_LENGTHS[27:35];
         endcase
      end
   endfunction // get_string_length

   function [0:7] get_pickup_location;
      input [0:1] which_string;
      begin
         case(which_string)
           2'b00: get_pickup_location = PICKUP_LOCATIONS[0:7];
           2'b01: get_pickup_location = PICKUP_LOCATIONS[8:15];
           2'b10: get_pickup_location = PICKUP_LOCATIONS[16:23];
           2'b11: get_pickup_location = PICKUP_LOCATIONS[24:31];
         endcase
      end
   endfunction // get_pickup_location

   function [0:15] get_pluck_value;
      input [0:1] which_string;
      begin
         case(which_string)
           2'b00: get_pluck_value = PLUCK_VALUES[0:15];
           2'b01: get_pluck_value = PLUCK_VALUES[16:31];
           2'b10: get_pluck_value = PLUCK_VALUES[32:47];
           2'b11: get_pluck_value = PLUCK_VALUES[48:63];
         endcase
      end
   endfunction // get_pluck_value

   function [0:8] get_pluck_location;
      input [0:1] which_string;
      begin
         case(which_string)
           2'b00: get_pluck_location = PLUCK_LOCATIONS[0:8];
           2'b01: get_pluck_location = PLUCK_LOCATIONS[9:17];
           2'b10: get_pluck_location = PLUCK_LOCATIONS[18:26];
           2'b11: get_pluck_location = PLUCK_LOCATIONS[27:35];
         endcase
      end
   endfunction // get_pluck_location
   

   /*
    * These tasks have the same idea as the functions above
    * the only difference is that they set values instead of 
    * getting them.  You can think of them as being basically
    * the same things as pre-processor macros in C.
    * 
    * An example usage:  
    * 
    *   reg [0:1] string_bits;
    *   reg [0:8] length;
    * 
    *   ...
    * 
    *   set_string_length(string_bits, length);
    */
   task set_string_length;
      input [0:1] which_string;
      input [0:8] new_length;
      begin
         case(which_string)
           2'b00: STRING_LENGTHS[0:8] <= new_length;
           2'b01: STRING_LENGTHS[9:17] <= new_length;
           2'b10: STRING_LENGTHS[18:26] <= new_length;
           2'b11: STRING_LENGTHS[27:35] <= new_length;
         endcase // case(which_string)
      end
   endtask // set_string_length
   
   task set_pickup_location;
      input [0:1] which_string;
      input [0:7] new_location;
      begin
         case(which_string)
           2'b00: PICKUP_LOCATIONS[0:7] <= new_location;
           2'b01: PICKUP_LOCATIONS[8:15] <= new_location;
           2'b10: PICKUP_LOCATIONS[16:23] <= new_location;
           2'b11: PICKUP_LOCATIONS[24:31] <= new_location;
         endcase // case(which_string)
      end
   endtask // set_pickup_location
   
   task set_pluck_value;
      input [0:1] which_string;
      input [0:15] new_value;
      begin
         case(which_string)
           2'b00: PLUCK_VALUES[0:15] <= new_value;
           2'b01: PLUCK_VALUES[16:31] <= new_value;
           2'b10: PLUCK_VALUES[32:47] <= new_value;
           2'b11: PLUCK_VALUES[48:63] <= new_value;
         endcase // case(which_string)
      end
   endtask // set_pluck_value
   
   task set_pluck_location;
      input [0:1] which_string;
      input [0:8] new_location;
      begin
         case(which_string)
           2'b00: PLUCK_LOCATIONS[0:8] <= new_location;
           2'b01: PLUCK_LOCATIONS[9:17] <= new_location;
           2'b10: PLUCK_LOCATIONS[18:26] <= new_location;
           2'b11: PLUCK_LOCATIONS[27:35] <= new_location;
         endcase // case(which_string)
      end
   endtask // set_pluck_location
   

   always @(posedge CLK)
     begin
        if(RST)
          begin
             CTRL_DATA_OUT <= 0;
             CTRL_ACK <= 0;
             MASTER_ENABLE <= 0;
             STRING_LENGTHS <= {4{9'd99}};
             PICKUP_LOCATIONS <= {4{8'd5}};
             PLUCK_VALUES <= {4{16'd30000}};
             PLUCK_LOCATIONS <= {4{9'd10}};
             PLUCK_STROBES <= 0;
             dummy_register <= 0;
          end // if (RST)
        else
          begin
             // TODO: Fill in this part so that it will update the
             // appropriate values or output the apropriate values based on the
             // address as well as set the ack and data out signals as
             // appropriate
          end // else: !if(RST)
     end // always @ (posedge CLK)
   
endmodule // simulation_control

                  
                    
                  
        
        

   