`timescale 1ns / 1ns

/* string_control
 * encapsulates everything you need for a string.  It takes
 * the various parameters and when you strobe the TRIGGER_SAMPLE_STROBE
 * it will start the process of generating a new sample.  The value
 * is then valid when the SAMPLE_VALID_STROBE strobes
 * 
 * Signals:
 * RST                    active high reset
 * 
 * CLK                    50MHz clock
 * 
 * TRIGGER_SAMPLE_STROBE  a strobe to signal that a new sample should be
 *                        generated.  It should be asserted for one clock
 *                        cycle.  The resulting sample will be valid when
 *                        SAMPLE_VALID_STROBE strobes.
 * 
 * PLUCK_STROBE           signal to pluck the string.  It should be strobed
 *                        for one clock cycle
 * 
 * PLUCK_LOCATION         where along the string to pluck.  This value gets
 *                        passed to the impulse generator to generate the 
 *                        pluck on this string.  It must be valid at the
 *                        time that PLUCK_STROBE goes high
 * 
 * PLUCK_VALUE            the value of the pluck to be passed to the 
 *                        impulse generator.  It must be valid at the time
 *                        that the PLUCK_STROBE is strobed
 * 
 * SIZE                   the length of the string.
 * 
 * PICKUP_LOC             where along the string to put the pickup location
 *                        it must be within 255 of either end of the string
 * 
 * SAMPLE                 the output of the next sample
 * 
 * SAMPLE_VALID_STROBE    signals that the sample output is valid.  This
 *                        only strobes for a single cycle, so make sure
 *                        you don't miss it.
 */
module string_control(RST, CLK,
					  
					  TRIGGER_SAMPLE_STROBE,
					  PLUCK_STROBE, PLUCK_LOCATION, PLUCK_VALUE,
					  SIZE, PICKUP_LOC, 
					  
					  SAMPLE, SAMPLE_VALID_STROBE);

   parameter STATE_READY             = 0;
   parameter STATE_STROBING_PLUCKERS = 1;
   parameter STATE_PLUCKING          = 2;
   parameter STATE_GETTING_SAMPLES   = 3;
   parameter STATE_MERGING_SAMPLES   = 4;
   parameter STATE_STROBING_VALID    = 5;
   
   
   input RST;
   input CLK;

   wire  RST;
   wire  CLK;


   
   input TRIGGER_SAMPLE_STROBE;
   wire  TRIGGER_SAMPLE_STROBE;


   
   input PLUCK_STROBE;
   input [0:8] PLUCK_LOCATION;
   input [0:15] PLUCK_VALUE;
   
   wire 		PLUCK_STROBE;
   wire [0:8] 	PLUCK_LOCATION;
   wire [0:15] 	PLUCK_VALUE;
   


   
   input [0:8] 	SIZE;
   input [0:7] 	PICKUP_LOC;
   
   wire [0:8] 	SIZE;
   wire [0:7] 	PICKUP_LOC;
   

   
   output 		SAMPLE_VALID_STROBE;
   output [0:15] SAMPLE;
   
   wire 		 SAMPLE_VALID_STROBE;
   reg [0:15] 	 SAMPLE, next_SAMPLE;


   

   // Internal signals
   reg [0:2] 	 state, next_state;
   reg [0:15] 	 l_sample, next_l_sample;
   reg [0:15] 	 r_sample, next_r_sample;
   

   
   // wires for the delay lines
   wire 		 lg_clk, rg_clk;
   wire 		 lg_rst, rg_rst;
   wire [0:8] 	 lg_size, rg_size;
   wire 		 lg_enable, rg_enable;
   wire [0:7] 	 lg_pickup_location, rg_pickup_location;
   wire [0:15] 	 lg_pickup_value, rg_pickup_value;
   wire [0:15] 	 lg_sample_in, rg_sample_in;
   wire [0:15] 	 lg_sample_out, rg_sample_out;

   
   // wires for the pluckers
   wire 		 lp_clk, rp_clk;
   wire 		 lp_rst, rp_rst;
   wire 		 lp_start_strobe, rp_start_strobe;
   wire 		 lp_busy, rp_busy;
   wire [0:8] 	 lp_size, rp_size;
   wire [0:8] 	 lp_peak_pos, rp_peak_pos;
   wire [0:15] 	 lp_peak_val, rp_peak_val;
   wire 		 lp_output_enable, rp_output_enable;
   wire [0:15] 	 lp_output, rp_output;
   
   
   // wires for the filters
   wire 		 bridge_clk, nut_clk;
   wire 		 bridge_rst, nut_rst;
   wire 		 bridge_enable, nut_enable;
   wire [0:15] 	 bridge_data_in, nut_data_in;
   wire [0:15] 	 bridge_data_out, nut_data_out;


   // assign for this module's interface
   assign 		 SAMPLE_VALID_STROBE = (state == STATE_STROBING_VALID);
   

   
   // delay line assigns  lg means left going, rg means 
   // right going as in wave
   assign 		 lg_clk = CLK;
   assign 		 lg_rst = RST;
   assign 		 lg_size = SIZE;
   assign 		 lg_enable = (state == STATE_PLUCKING) ? lp_output_enable : (state == STATE_GETTING_SAMPLES);
   assign 		 lg_pickup_location = PICKUP_LOC;
   assign 		 lg_sample_in = (state == STATE_PLUCKING) ? lp_output : bridge_data_out;   

   assign 		 rg_clk = CLK;
   assign 		 rg_rst = RST;
   assign 		 rg_size = SIZE;
   assign 		 rg_enable = (state == STATE_PLUCKING) ? rp_output_enable : (state == STATE_GETTING_SAMPLES);
   assign 		 rg_pickup_location = SIZE - PICKUP_LOC;
   assign 		 rg_sample_in = (state == STATE_PLUCKING) ? rp_output : nut_data_out;   


   // plucker assigns
   assign 		 lp_clk = CLK;
   assign 		 lp_rst = RST;
   assign 		 lp_start_strobe = (state == STATE_STROBING_PLUCKERS);
   assign 		 lp_size = SIZE;
   assign 		 lp_peak_pos = PLUCK_LOCATION;
   assign 		 lp_peak_val = PLUCK_VALUE;
   
   assign 		 rp_clk = CLK;
   assign 		 rp_rst = RST;
   assign 		 rp_start_strobe = (state == STATE_STROBING_PLUCKERS);
   assign 		 rp_size = SIZE;
   assign 		 rp_peak_pos = SIZE - PLUCK_LOCATION;
   assign 		 rp_peak_val = PLUCK_VALUE;
   

   // filter assigns
   assign 		 bridge_clk = CLK;
   assign 		 bridge_rst = RST;
   assign 		 bridge_enable = (state == STATE_GETTING_SAMPLES);
   assign 		 bridge_data_in = rg_sample_out;

   assign 		 nut_clk = CLK;
   assign 		 nut_rst = RST;
   assign 		 nut_enable = (state == STATE_GETTING_SAMPLES);
   assign 		 nut_data_in = lg_sample_out;
   
				 


   
   always @(posedge CLK)
	 begin
		if(RST)
		  begin
			 state <= STATE_READY;
			 SAMPLE <= 0;
			 l_sample <= 0;
			 r_sample <= 0;
		  end
		else
		  begin
			 SAMPLE <= next_SAMPLE;
			 state <= next_state;
			 l_sample <= next_l_sample;
			 r_sample <= next_r_sample;
		  end
	 end // always @ (posedge CLK)


   // the state machine  This just goes between states
   always @(*)
	 begin
		next_state = state;
		case(state)
		  STATE_READY:
			begin
			   // note that pluck takes precedence
			   if(TRIGGER_SAMPLE_STROBE) next_state = STATE_GETTING_SAMPLES;
			   if(PLUCK_STROBE) next_state = STATE_STROBING_PLUCKERS;
			end
		  STATE_STROBING_PLUCKERS:
			begin
			   next_state = STATE_PLUCKING;
			end
		  STATE_PLUCKING:
			begin
			   if(!(lp_busy || rp_busy)) next_state = STATE_READY;
			end
		  STATE_GETTING_SAMPLES:
			begin
			   next_state = STATE_MERGING_SAMPLES;
			end
		  STATE_MERGING_SAMPLES:
			begin
			   next_state = STATE_STROBING_VALID;
			end
		  STATE_STROBING_VALID:
			begin
			   next_state = STATE_READY;
			end
		endcase // case(state)
	 end // always @ (*)

   
	
   always @(*)
	 begin
		next_SAMPLE = SAMPLE;
		if(state == STATE_MERGING_SAMPLES)
		  begin
			 next_SAMPLE = l_sample + r_sample;
		  end
	 end


   
   always @(*)
	 begin
		next_l_sample = l_sample;
		next_r_sample = r_sample;
		if(state == STATE_GETTING_SAMPLES)
		  begin
			 next_l_sample = lg_pickup_value;
			 next_r_sample = rg_pickup_value;
		  end
	 end





   delay_line_with_pickup lg_line
	 (
	  .CLK(lg_clk),
	  .RST(lg_rst),
	  .SIZE(lg_size),
	  .ENABLE(lg_enable),
	  .PICKUP_LOC(lg_pickup_location),
	  .PICKUP_VALUE(lg_pickup_value),
	  .SAMPLE_IN(lg_sample_in),
	  .SAMPLE_OUT(lg_sample_out)
	  );

   delay_line_with_pickup rg_line
	 (
	  .CLK(rg_clk),
	  .RST(rg_rst),
	  .SIZE(rg_size),
	  .ENABLE(rg_enable),
	  .PICKUP_LOC(rg_pickup_location),
	  .PICKUP_VALUE(rg_pickup_value),
	  .SAMPLE_IN(rg_sample_in),
	  .SAMPLE_OUT(rg_sample_out)
	  );
   
   impulse_generator l_plucker
	 (
	  .CLK(lp_clk),
	  .RST(lp_rst),
	  .START_STROBE(lp_start_strobe),
	  .BUSY(lp_busy),
	  .SIZE(lp_size),
	  .PEAK_POS(lp_peak_pos),
	  .PEAK_VAL(lp_peak_val),
	  .OUTPUT_ENABLE(lp_output_enable),
	  .OUTPUT(lp_output)
	  );

   impulse_generator r_plucker
	 (
	  .CLK(rp_clk),
	  .RST(rp_rst),
	  .START_STROBE(rp_start_strobe),
	  .BUSY(rp_busy),
	  .SIZE(rp_size),
	  .PEAK_POS(rp_peak_pos),
	  .PEAK_VAL(rp_peak_val),
	  .OUTPUT_ENABLE(rp_output_enable),
	  .OUTPUT(rp_output)
	  );

   bridge_filter bf
	 (
	  .CLK(bridge_clk),
	  .RST(bridge_rst),
	  .ENABLE(bridge_enable),
	  .DATA_IN(bridge_data_in),
	  .DATA_OUT(bridge_data_out)
	  );
   
   nut_filter nf
	 (
	  .CLK(nut_clk),
	  .RST(nut_rst),
	  .ENABLE(nut_enable),
	  .DATA_IN(nut_data_in),
	  .DATA_OUT(nut_data_out)
	  );
   

endmodule // string_control

	  
		




   