`timescale 1ns / 1ns

/** tf_bridge_filter
 * A test fixture for the bridge filter.  The clock runs forever, but
 * the tests only last for a little while.  You'll want to
 * look at the signals in the simulator to see if it's working
 * correctly
 */

module tf_bridge_filter();

   reg CLK;
   reg RST;
   reg ENABLE;
   reg [0:15] DATA_IN;
   wire [0:15] DATA_OUT;


   reg 		   some_stages_failed;


   task verify_DATA_OUT;
	  input [0:9] stage_number;
	  input [0:15] expected_value;
	  begin
		 if(DATA_OUT == expected_value)
		   begin
			  $display("Stage %d passed", stage_number);
		   end
		 else
		   begin
			  $display("Stage %d failed", stage_number);
			  $display("Expected %h, but got %h",
					   expected_value,
					   DATA_OUT);
			  some_stages_failed = 1;
		   end // else: !if(DATA_OUT == expected_value)
	  end
   endtask // verify_DATA_OUT
   

   // generate the reset signal
   initial
	 begin
		RST = 1;
		#50; // wait for 2.5 clock cycles with reset high
		RST = 0;
	 end


   // generate the clock signal
   initial
	 begin
		CLK = 1;
		while(1)
		  begin
			 #10;
			 CLK = ~CLK;
		  end
	 end


   

   // generate the data elements with which to test
   initial
	 begin
		some_stages_failed = 0;
		ENABLE = 0;
		DATA_IN = 16'h0000;
		// simulate some delay in the enable signal assertion
		#80; // let the system settle out of the reset state

		// now, we can start giving it data, remember that
		// verilog does not have negatives, so you have to deal
		// with the 2's compliment numbers yourself

		// stage 1
		DATA_IN = 16'h0200;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd1, 16'h0000);
	
		#80; // wait a few clocks for things to settle.  In the
		     // real circuit, this will probably be about 1000 cycles,
		     // but that probably doesn't affect our testing
	
		// stage 2
		DATA_IN = 16'h0246;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd2, 16'hff02);
  
		#80;
		
		// stage 3
		DATA_IN = 16'h0248;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd3, 16'hfe60);

		#80; 	

		// stage 4
		DATA_IN = 16'h0000;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd4, 16'hfdcf);
  
		#80;
		
		// stage 5
		DATA_IN = 16'h1256;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd5, 16'hfedf);

		#80; 	

		// stage 6
		DATA_IN = 16'h8034;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd6, 16'hf656);
  
		#80;
		
		// stage 7
		DATA_IN = 16'hfff3;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd7, 16'h3ada);

		#80; 	

		// stage 8
		DATA_IN = 16'h8925;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd8, 16'h1b2d);
  
		#80;
		
		// stage 9
		DATA_IN = 16'h3683;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd9, 16'h5aad);

		#80; 	

		// stage 10
		DATA_IN = 16'hfac3;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd10, 16'h0274);

		#80; 	

		// stage 11
		DATA_IN = 16'habc2;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd11, 16'h128f);

		#80; 	

		// stage 12
		DATA_IN = 16'hedc7;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd12, 16'h1d92);

		#80; 	

		// stage 13
		DATA_IN = 16'h2513;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd13, 16'h1f3d);

		#80; 	

		// stage 14
		DATA_IN = 16'h0000;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd14, 16'h0706);

		#80; 	
		// stage 15
		DATA_IN = 16'h0000;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd15, 16'hfb53);

		#80; 	
		// stage 16
		DATA_IN = 16'h0000;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd16, 16'hf6ce);

		#80; 	
		// stage 17
		DATA_IN = 16'h0000;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
		verify_DATA_OUT(10'd17, 16'h0000);

		#80; 	


		if(some_stages_failed)
		  begin
			 $display("Some stages failed");
		  end
		else
		  begin
			 $display("All stages passed");
		  end
	 end // initial begin

		
   
   bridge_filter bf_under_test
	 (
	  .CLK(CLK),
	  .RST(RST),
	  .ENABLE(ENABLE),
	  .DATA_IN(DATA_IN),
	  .DATA_OUT(DATA_OUT)
	  );

endmodule // tf_bridge_filter

