`timescale 1ns / 1ns

/** 
 * A bunch of hardcoded stimulus and expected responses
 * for the mixer
 */
module tf_mixer();

   reg         CLK;
   reg         RST;
   
   wire [0:15] LEFT_CHANNEL;
   wire [0:15] RIGHT_CHANNEL;
   wire 	   OUTPUT_VALID_STROBE;


   wire [0:15] STRING0, STRING1, STRING2, STRING3;
   wire 	   STRING0_VALID_STROBE, STRING1_VALID_STROBE,
			   STRING2_VALID_STROBE, STRING3_VALID_STROBE;
   
   reg [0:15]  st0, st1, st2, st3;
   reg 		   sv0, sv1, sv2, sv3;

   reg 		   awaiting_response;
   reg 		   spurious_oa;
   reg 		   some_tests_failed;
   

   task verify_response;
	  input [0:9] test_number;
	  input [0:15] expected_left;
	  input [0:15] expected_right;
	  begin		 
		 awaiting_response = 1;
		 while(OUTPUT_VALID_STROBE == 1'b0) #20;
		 awaiting_response = 0;
		 // at this point, I should be able to look at the outputs
		 // and they'd better be right
		 if({LEFT_CHANNEL, RIGHT_CHANNEL} == {expected_left, expected_right})
		   begin
			  $display("Test %d passed", test_number);
		   end
		 else
		   begin
			  $display("Test %d failed", test_number);
			  $display("Expected {LEFT, RIGHT} %h, %h, but got %h, %h",
					   expected_left, expected_right, 
					   LEFT_CHANNEL, RIGHT_CHANNEL);
			  some_tests_failed = 1;
		   end
	  end
   endtask // verify_response
   
	  
	  
	  


   assign 	   STRING0 = st0;
   assign 	   STRING1 = st1;
   assign 	   STRING2 = st2;
   assign 	   STRING3 = st3;
   assign 	   STRING0_VALID_STROBE = sv0;
   assign 	   STRING1_VALID_STROBE = sv1;
   assign 	   STRING2_VALID_STROBE = sv2;
   assign 	   STRING3_VALID_STROBE = sv3;
   

   initial
	 begin
		RST = 1;
		#50;
		RST = 0;
	 end


   initial
	 begin
		CLK = 1;
		while(1)
		  begin
			 #10;
			 CLK = ~CLK;
		  end
	 end


   initial
	 begin
		{st0, st1, st2, st3} = {16'h0000, 16'h0000, 16'h0000, 16'h0000};
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		awaiting_response = 0;
		some_tests_failed = 0;
		
		#80; // let things settle


		
		// first test all zeroes
		{st0, st1, st2, st3} = {16'h0000, 16'h0000, 16'h0000, 16'h0000};
		{sv0, sv1, sv2, sv3} = { 1'b1,     1'b1,     1'b1,     1'b1};
		#20;
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		verify_response(10'd1, 16'h0000, 16'h0000);
		
		#40;  // this would normally be 1000 cycles, but we can make it
		      // 2 for testing.

		// second test small positives
		{st0, st1, st2, st3} = {16'h0022, 16'h0058, 16'h004c, 16'h0037};
		{sv0, sv1, sv2, sv3} = { 1'b1,     1'b1,     1'b1,     1'b1};
		#20;
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		verify_response(10'd2, 16'h003a, 16'h0043);
		
		#40;  // this would normally be 1000 cycles, but we can make it
		      // 2 for testing.


		// third test large positives
		{st0, st1, st2, st3} = {16'h73ab, 16'h35ff, 16'h5264, 16'h1234};
		{sv0, sv1, sv2, sv3} = { 1'b1,     1'b1,     1'b1,     1'b1};
		#20;
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		verify_response(10'd3, 16'h5a27, 16'h2cf9);
		
		#40;  // this would normally be 1000 cycles, but we can make it
		      // 2 for testing.


		// fourth test small negatives
		{st0, st1, st2, st3} = {16'hfff2, 16'hfff8, 16'hff3c, 16'hff37};
		{sv0, sv1, sv2, sv3} = { 1'b1,     1'b1,     1'b1,     1'b1};
		#20;
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		verify_response(10'd4, 16'hffd1, 16'hff5c);
		
		#40;  // this would normally be 1000 cycles, but we can make it
		      // 2 for testing.


		// fifth test large negatives
		{st0, st1, st2, st3} = {16'h8046, 16'habcd, 16'h925c, 16'h9999};
		{sv0, sv1, sv2, sv3} = { 1'b1,     1'b1,     1'b1,     1'b1};
		#20;
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		verify_response(10'd5, 16'h9144, 16'h9abf);
		
		#40;  // this would normally be 1000 cycles, but we can make it
		      // 2 for testing.


		// sixth test mixed
		{st0, st1, st2, st3} = {16'h47ad, 16'hbae3, 16'h1ee7, 16'h2637};
		{sv0, sv1, sv2, sv3} = { 1'b1,     1'b1,     1'b1,     1'b1};
		#20;
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		verify_response(10'd6, 16'h1408, 16'h0fce);
		
		#40;  // this would normally be 1000 cycles, but we can make it
		      // 2 for testing.


		// seventh test mixed
		{st0, st1, st2, st3} = {16'h2722, 16'hdf38, 16'h15ab, 16'h2fda};
		{sv0, sv1, sv2, sv3} = { 1'b1,     1'b1,     1'b1,     1'b1};
		#20;
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		verify_response(10'd7, 16'h0d62, 16'h188c);
		
		#40;  // this would normally be 1000 cycles, but we can make it
		      // 2 for testing.


		// eighth test small positives
		{st0, st1, st2, st3} = {16'h8000, 16'h8001, 16'h8001, 16'h8001};
		{sv0, sv1, sv2, sv3} = { 1'b1,     1'b1,     1'b1,     1'b1};
		#20;
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		verify_response(10'd8, 16'h8000, 16'h8001);
		
		#40;  // this would normally be 1000 cycles, but we can make it
		// 2 for testing.
		
		
		// ninth test mixed
		{st0, st1, st2, st3} = {16'hbad2, 16'hd358, 16'hfeaa, 16'h3737};
		{sv0, sv1, sv2, sv3} = { 1'b1,     1'b1,     1'b1,     1'b1};
		#20;
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		verify_response(10'd9, 16'hcf34, 16'h12d1);
		
		#40;  // this would normally be 1000 cycles, but we can make it
		      // 2 for testing.

		// tenth test mixed
		{st0, st1, st2, st3} = {16'h68b2, 16'h15a8, 16'h263c, 16'h0037};
		{sv0, sv1, sv2, sv3} = { 1'b1,     1'b1,     1'b1,     1'b1};
		#20;
		{sv0, sv1, sv2, sv3} = { 1'b0,     1'b0,     1'b0,     1'b0};
		verify_response(10'd10, 16'h4248, 16'h101d);
		
		#40;  // this would normally be 1000 cycles, but we can make it
		      // 2 for testing.


		

		if(some_tests_failed || spurious_oa)
		  begin
			 $display("FINISHED, some tests failed");
		  end
		else
		  begin
			 $display("FINISHED, all tests passed");

		  end
	 end




   // this is on the negedge to get it to sample at the
   // right time
   always @(negedge CLK)
	 begin
		if(RST)
		  begin
			 spurious_oa = 0;
		  end
		else
		  begin
			 if(!awaiting_response && OUTPUT_VALID_STROBE)
			   begin
				  $display("ERROR: spurious output_valid_strobe");
				  spurious_oa = 1;
			   end
		  end
	 end // always @ (negedge CLK)
   
   
		 
   
   

   mixer m
	 (
	  .CLK(CLK),
	  .RST(RST),
	  .LEFT_CHANNEL(LEFT_CHANNEL),
	  .RIGHT_CHANNEL(RIGHT_CHANNEL),
	  .OUTPUT_VALID_STROBE(OUTPUT_VALID_STROBE),
	  .STRING0(STRING0),
	  .STRING1(STRING1),
	  .STRING2(STRING2),
	  .STRING3(STRING3),
	  .STRING0_VALID_STROBE(STRING0_VALID_STROBE),
	  .STRING1_VALID_STROBE(STRING1_VALID_STROBE),
	  .STRING2_VALID_STROBE(STRING2_VALID_STROBE),
	  .STRING3_VALID_STROBE(STRING3_VALID_STROBE)
	  );
   
	  
endmodule // tf_mixer


		